/*
 *
 *	Averigua en tiempo de ejecucin qu biblioteca debe usar
 *	para procesar un archivo.
 *      Usa la tcnica de la carga explcita de bibliotecas dinmicas
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <dlfcn.h>
#include <string.h>
#include <stdlib.h>

/* Busca en el archivo de configuracin qu biblioteca est asociada
   a la extensin especificada. Devuelve NULL si no la encuentra */

static char *buscar_biblio_en_config(char *extension_buscada) {
	FILE *fconfig;
	char *biblio;
	char extension_config[32];

	biblio=malloc(128*sizeof(char));

	if ((fconfig=fopen("config", "r"))==NULL) {
		perror("No puede abrir archivo de configuracin");
		exit(1);
	}
	while (fscanf(fconfig, "%s %s", extension_config, biblio)!=EOF) 
		if (strcmp(extension_config, extension_buscada)==0) {
			fclose(fconfig);
			return biblio;
		}

	fclose(fconfig);
	return NULL;
}

/* Dado un nombre de archivo devuelve su extensin. Retorna NULL si el
   nombre no tiene extensin o el punto es el ltimo carcter. */
static char *obtener_extension(char *archivo) {
		char *ext;

		ext=rindex(archivo, '.');
		/* Si hay un '.', ext != NULL */
		if (ext!=NULL) {
			/* se avanza despus del punto */
			ext++;
			/* Si '.' es ltimo car., no hay realmente extensin */
			if (*ext=='\0')
				ext=NULL;
		}
		return ext;
}

/* Procesa el archivo usando la biblioteca dinmica especificada */
static void tratar_archivo(char *archivo, char *biblioteca){
	void *descriptor_bib;
	void (*procesar)(char *);

	/* Se carga la biblioteca dinmica */
	if (!(descriptor_bib=dlopen(biblioteca, RTLD_LAZY))) {
		fprintf(stderr, "Error cargando biblioteca: %s\n", dlerror());
		return;
	}

	/* Busca el smbolo correspondiente a "procesar" */
	if (!(procesar=dlsym(descriptor_bib, "procesar")))
		/* La funcin no existe */
                fprintf(stderr, "Error: biblioteca no incluye funcin procesar\n");
	else
		/* Finalmente, procesamos el archivo */
		procesar(archivo);

	/* Se descarga la biblioteca */
	dlclose(descriptor_bib);
}

int main() {
	char archivo[256];
	char *extension;
	char *biblioteca;

	printf("Introduzca el nombre del archivo que quiere procesar: ");
	while (scanf("%s", archivo)!=EOF) {
		/* Obtiene la extensin correspondiente al nombre */		
		extension=obtener_extension(archivo);
		if (extension==NULL)
			fprintf(stderr, "Error: el nombre de archivo debe tener extensin\n");
		else {
			/* Busca la bobliteca asociada */
			biblioteca=buscar_biblio_en_config(extension);
			if (biblioteca==NULL)
				fprintf(stderr, "Error: extensin no configurada\n");
			else
				/* procesa el archivo usando la bibloteca */
				tratar_archivo(archivo, biblioteca);
		}

		printf("Introduzca el nombre del archivo que quiere procesar: ");
	}
	return 0;
}
