/*
   Programa que transforma las letras minscuslas que haya en un
   archivo a maysculas usando la tcnica de proyeccin de archivos.
   Si slo recibe un nombre de archivo, la transformacin se hace "in-situ".
   Si recibe dos nombres, deja la transformacin realizada sobre el primero
   en el segundo.

*/


#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>

int main(int  argc, char **argv) {
	int i, fd, fdd;
	int flags_open, flags_mmap, prot_mmap;
	char *org, *dst;
	struct stat bstat;
	int insitu; /* Se hace la transformacin "insitu"? */

	if ((argc!=2) && (argc!=3)) {
		fprintf (stderr, "Uso: %s archivo [archivo_destino]\n", argv[0]);
		return(1);
	}

	/* Si no hay archivo destino, la transformacin se hace "insitu":
		El archivo origen debe abrirse en modo lect/escr y
		proyectarse de forma compartida y con permisos de lect/escr.

	   Si hay archivo destino, la transformacin no se hace "insitu":
		El archivo origen debe abrirse en modo lectura y
		proyectarse de forma privada con permisos de lectura. 
	 */
	if (argc==2) {
		insitu=1;
		flags_open=O_RDWR;
		flags_mmap=MAP_SHARED;
		prot_mmap=PROT_READ|PROT_WRITE;
	}
	else {
		insitu=0;
		flags_open= O_RDONLY;
		flags_mmap=MAP_PRIVATE;
		prot_mmap=PROT_READ;
	}

	/* Abre el archivo origen */
	if ((fd=open(argv[1], flags_open))<0) {
		perror("No puede abrirse el archivo");
		return(1);
	}

	/* Averigua la longitud del archivo */
	if (fstat(fd, &bstat)<0) {
		perror("Error en fstat del archivo");
		close(fd);
		return(1);
	}

	/* Se proyecta el archivo origen */
	if ((org=mmap((caddr_t) 0, bstat.st_size, prot_mmap,
			flags_mmap, fd, 0)) == (void *)MAP_FAILED) {
		perror("Error en la proyeccion del archivo");
		close(fd);
		return(1);
	}

	/* Se cierra el archivo */
	close(fd);

	/* Si no se trabaja "insitu", se abre y proyecta archivo destino */
	if (!insitu) {
		/* Abre el archivo destino */
		if ((fdd=open(argv[2], O_CREAT|O_TRUNC|O_RDWR, 0640))<0) {
			perror("No puede crearse el archivo destino");
			return(1);
		}
		/* Establece longitud archivo destino igual a la del origen.*/
		if (ftruncate(fdd, bstat.st_size)<0) {
			perror("Error en ftruncate del archivo destino");
                	close(fdd);
			unlink (argv[2]);
			return(1);
        	}
		/* Se proyecta el archivo destino */
		if ((dst=mmap((caddr_t) 0, bstat.st_size, PROT_WRITE,
				MAP_SHARED, fdd, 0)) == MAP_FAILED) {
			perror("Error en la proyeccion del archivo destino");
			close(fdd);
			unlink (argv[2]);
			return(1);
        	}
        }

	/* puntero auxiliar para recorrer destino */
	if (insitu)
		/* si "insitu" el puntero destino apunta a archivo origen */
		dst=org;
	else
		/* si no el puntero destino apunta a archivo destino */
		dst=dst;

	/* Bucle de acceso que sustituye las minsculas por maysculas */
	for (i=0; i<bstat.st_size; i++)
		if (islower(*org))
			*dst=toupper(*org);
		else
			*dst=*org;

	/* Se eliminan las proyecciones */
	munmap(org, bstat.st_size);

	if (!insitu)
		munmap(dst, bstat.st_size);

	return(0);
}
